/* cscreen.h
 * --  this is implementation of an art of 'self-freeing' stack, that
 * frees old data by overwriting them with the new one (with cscreen_write).
 * Once data readed (with cscreen_read) they will became unavaible - So you
 * read always data from the 'top' and write them to the 'bottom' of buffer.
 * Writing of data is very fast - there are used only two memcpy() calls.
 * NOTE: This way of storing current screen image is very unreliable !!!
 */

#ifndef __CSCREEN_H
#define __CSCREEN_H

/* screen cache structure */
typedef struct
{
    char *buffer;
    unsigned int size;
    unsigned int start;
    unsigned int load;
}
cscreen_t;

/* helper functions */
#define cscreen_size(csc)	(unsigned int) (csc)->size
#define cscreen_load(csc)	(unsigned int) (csc)->load
#define cscreen_buffer(csc)	(const char *) (csc)->buffer

/* Create and initialize new cscreen_t structure and return its pointer
 * or NULL on error */
cscreen_t *cscreen_open(unsigned int max_length);

/* resizes buffer size of the cscreen struture, on error cscreen_t is left
 * untouched and -1 is returned */
int cscreen_resize(cscreen_t * csc, unsigned int new_length);

/* writes buffer to cscreen_t structure, returns number of writen chars
 * -- size of input buffer is not limited */
int cscreen_write(cscreen_t * csc, register const char *buffer,
		  unsigned int length);

/* reads data from buffer and expires them, returns number of readed chars
 * or -1 if no more data avaible */
int cscreen_read(cscreen_t * csc, char *buffer, unsigned int length);

/* prints out whole data to specified file without expiring them, returns
 * -1 if data could not be written to file else returns its size */
int cscreen_print(cscreen_t * csc, register int f);

/* clears and resets cscreen_t cache */
int cscreen_clear(cscreen_t * csc);

/* free cscreen_t structure */
int cscreen_close(cscreen_t * csc);

#endif
