/* pam_client.c  - part of pam_watch.c
 * AUTHOR               : Samuel Behan <behan@frida.fri.utc.sk> (c) 2001
 * LICENSE      : GNU GPL v2 or later, NO WARANTY VOID provided
 * NOTE         : Modified by Samule Behan <behan@frida.fri.utc.sk> 
 * from original pam_filter by Andrew Morgan <morgan@transmeta.com> 
 * and (his) Richard Stevens' UNIX Network Programming book.
 */

#include <stdio.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <termios.h>

#include <security/pam_modules.h>

#include "pam_watch.h"


int start_client(pam_handle_t * pamh, int fd[2],
		 struct termios old_mode, watch_config_t * watch)
{
    struct winsize win =
    {0, 0, 0, 0};

    /* get current winsize */
    if (ioctl(STDOUT_FILENO, TIOCGWINSZ, &win) == -1)
    {
	ON_DEBUG(_pam_log(LOG_WARNING, "can not get current window size"));
	/* we don need to abort this is 'not' very important to terminal */
    }
    /* setup slave side interlink */
    close(fd[0]);
    if (dup2(fd[1], STDIN_FILENO) == -1
	|| dup2(fd[1], STDOUT_FILENO) == -1
	|| dup2(fd[1], STDERR_FILENO) == -1)
    {
	ON_DEBUG(_pam_log(LOG_WARNING, "can not assign STDIN/OUT/ERR"));
	return WATCH_IOERR;
    }
    close(fd[1]);
    if (fcntl(STDIN_FILENO, F_SETFD, 0) == -1
	|| fcntl(STDOUT_FILENO, F_SETFD, 0) == -1
	|| fcntl(STDERR_FILENO, F_SETFD, 0) == -1)
    {
	ON_DEBUG(_pam_log(LOG_WARNING, "unable to retain I/O modes"));
	return WATCH_IOERR;
    }
    /* become new session leader */
    if (setsid() == -1)
    {
	ON_DEBUG(_pam_log(LOG_WARNING,
			  "child can not became new session leader"));
	return WATCH_PROCERR;
    }
    /* get our control terminal, back */
    if (ioctl(STDIN_FILENO, TIOCSCTTY, STDIN_FILENO) == -1)
    {
	ON_DEBUG(_pam_log(LOG_WARNING,
		       "child can not set controling terminal, ignoring"));
    }
    /* put user terminal to normal mode */
    if (tcsetattr(STDIN_FILENO, TCSANOW, &old_mode) == -1)
    {
	ON_DEBUG(_pam_log(LOG_WARNING, "can not set back terminal mode"));
	return WATCH_IOERR;
    }
    /* re-set winsize */
    if (win.ws_col != 0 && win.ws_row != 0)
	if (ioctl(STDOUT_FILENO, TIOCSWINSZ, &win) == -1)
	{
	    ON_DEBUG(_pam_log
		     (LOG_WARNING, "can not get current window size"));
	    /* we don need to abort this is not very important to terminal */
	}
    return WATCH_SUCCESS;
}
