/*pam_watch.h
 * -- header file for pam_watch module (pam_watch.c)
 */

#ifndef __PAM_WATCH_H
#define __PAM_WATCH_H

/* module arguments indicators  */
#define WATCH_DEBUG	0001	/* enable debuging messages */
#define WATCH_PATH	0002	/* path argument (required) */
#define WATCH_SPY	0004	/* spy users|groups */
#define WATCH_NOSPY	0010	/* nospy users|groups */
#define WATCH_NOWRITE	0020	/* disable input */
#define WATCH_NOSUSPEND	0040	/* disable suspend feature (EXPERIMENTAL) */
#define WATCH_CACHE	0100	/* specify cache size */
#define WATCH_NOCACHE	0200	/* disable cache */
#define NEW_TERM	1000	/* generic arguments... */
#define NON_TERM	2000
#define NO_WARN		4000

/*want to drop privileges ??? */
#define DROP_PRIVILEGES
#define DROP_PAM_HANDLE
#define CATCH_SIGSEGV

/* suspend command activator -- need to fix (EXPERIMENTAL) */
#define SUSPEND_STRING "\033[s"
/* suspended mode - remote terminal resize initiator */
#define SUSPEND_RESIZE "\033[S"

/* screen cache defaults */
#define CSCREEN_MAX	19200
#define CSCREEN_DEFAULT  9600

/*buffer size */
#ifdef BUFSIZ
#undef BUFSIZ
#endif
#define BUFSIZ 4096

typedef struct
{
    char *path;
    char *spyspec;
    char *ofifo_name;
    char *ififo_name;
    char *propagate_name;
    char ctrl;
    int input_fifo;
    int output_fifo;
    int disable_input:1;
    int echo_input:1;
    int require_io:1;
    int suspended:1;
    int cache_enabled:1;
    unsigned int cache_size;
    pid_t child_pid;
}
watch_config_t;

#define watch_config_default(arg) arg.input_fifo = arg.output_fifo = -1; \
	arg.path = arg.spyspec = NULL; \
	arg.ofifo_name = arg.ififo_name = arg.propagate_name = NULL; \
	arg.ctrl = arg.disable_input = arg.require_io = arg.suspended = 0; \
	arg.echo_input	= arg.child_pid	= arg.cache_size	= 0; \
	arg.cache_enabled	= 1;

/* developer messages mgment */
#if defined(PAM_DEBUG) || defined(DEBUG)

#define ON_DEVEL(code) code
#define msg(msg) _pam_log(LOG_DEBUG, "[%s:%s(%d)] : %s", __FILE__, __FUNCTION__, __LINE__, (msg));
#else
#define ON_DEVEL(code)
#define msg(msg)
#endif

/*watch retvals*/
#define WATCH_SUCCESS	PAM_SUCCESS		/*oki*/
#define WATCH_ERROR	PAM_ABORT		/*unpredictable error*/
#define WATCH_BADCFG	PAM_AUTHINFO_UNAVAIL	/*bad configuration*/
#define WATCH_EGAIN	PAM_PERM_DENIED		/*permision error*/
#define WATCH_IOERR	PAM_ABORT		/*IO error (write, ioctl..)*/
#define WATCH_OPENERR	PAM_ABORT		/*open... failed*/
#define WATCH_PROCERR	PAM_SYSTEM_ERR		/*proccess error (fork()..)*/
#define WATCH_MEMERR	PAM_BUF_ERR		/*memory error (malloc..)*/


/* user debug messages */
#define ON_DEBUG(code) if(watch->ctrl & WATCH_DEBUG) code

/* system logger */
void _pam_log(int err, const char *format,...);

/* client and server prototypes */
int start_client(pam_handle_t * pamh, int fd[2], struct termios old_mode,
		 watch_config_t * watch);
int start_server(pam_handle_t * pamh, int fd[2], struct termios old_mode,
		 watch_config_t * watch_ref);

#endif
